/***   IOread.c - read input etc   ***/

#include <stdio.h>
#include <strings.h>
#include <math.h>
#include "header.h"
#include "Xdefs.h"

/* Read the command line */
fileopen(argc,argv)
int argc;
char **argv;
{ 
   FILE     *tmpfile, *optionfile, *fopen();
   char     *strcpy(), *strncpy();
   double   atof();
   pfileptr insert_pfile();
   extern   char   printer[MAXCHAR];
   extern   double scale;
   extern   int    line, marker, linechange, markerchange, plottypes;
   extern   int    equalscale, grid, landscape, postscript, xlog, ylog;
   extern   int    clip, printplot, noplot;
   extern   int    autorange;
   int      i,FILEFOUND;

   /* initialize general X arguments */
   progname     = argv[0];
   font_name    = NULL;
   geometry     = NULL;
   display_name = NULL;
   border_color = NULL;
   fore_color   = NULL;
   back_color   = NULL;
   mouse_color  = NULL;
   icon_color   = NULL;
   reverse      = 0;
   border_width = 0;

   FILEFOUND = FALSE;
   for (i=1; i < argc; i++) {     /* Parse Line */
      if (argv[i][0] == '=') {
         geometry = argv[i];
         continue;
         }
      if (strncmp(argv[i],"-geom",5)==0) {        /* new geometry */
         if (++i >= argc) usage(progname);
         geometry = argv[i];
         continue;
         }
      if (index(argv[i],':')!=NULL) {             /* host:display */
         strncpy(display_name,argv[i],sizeof(display_name));
         continue;
         }
      if (!strncmp(argv[i],"-d",2)) {             /* new display */
         if (++i >= argc) usage(progname);
         display_name = argv[i]; 
         continue;
         }
      if (strcmp(argv[i],"-rv") == 0) {           /* reverse display */
         reverse = 1;
         continue;
         }
      if (strcmp(argv[i],"-bw") == 0) {           /* border width */
         if (++i >= argc) usage(progname);
         border_width = atoi(argv[i]);
         continue;
         }
      if (strcmp(argv[i],"-bd") == 0) {           /* border color */
         if (++i >= argc) usage(progname);
         border_color = argv[i];
         continue;
         }
      if (strcmp(argv[i],"-fg") == 0) {           /* foreground color */
         if (++i >= argc) usage(progname);
         fore_color = argv[i];
         continue;
         }
      if (strcmp(argv[i],"-bg") == 0) {           /* background color */
         if (++i >= argc) usage(progname);
         back_color = argv[i];
         continue;
         }
      if (strcmp(argv[i],"-fn") == 0) {           /* font type */
         if (++i >= argc) usage(progname);
         font_name  = argv[i];
         continue;
         }
      if (strcmp(argv[i],"-xlog") == 0) {         /* x-log scale */
         xlog = ON;
         fprintf(stdout,"   Commands: xlog            = ON\n");
         continue;
         }
      if (strcmp(argv[i],"-ylog") == 0) {         /* y-log scale */
         ylog = ON;
         fprintf(stdout,"   Commands: ylog            = ON\n");
         continue;
         }
      if (strcmp(argv[i],"-ar") == 0) {         /* equalscale */
         autorange = OFF;
         fprintf(stdout,"   Commands: autorange       = OFF\n");
         continue;
         }
      if (strcmp(argv[i],"-clip") == 0) {         /* equalscale */
         clip = ON;
         fprintf(stdout,"   Commands: clip            = ON\n");
         continue;
         }
      if (strcmp(argv[i],"-e") == 0) {            /* equalscale */
         equalscale = ON;
         fprintf(stdout,"   Commands: equalscale      = ON\n");
         continue;
         }
      if (strcmp(argv[i],"-g") == 0) {            /* grid */
         grid = ON;
         fprintf(stdout,"   Commands: grid            = ON\n");
         continue;
         }
      if (strcmp(argv[i],"-l") == 0) {            /* landscape */
         landscape = ON;
         fprintf(stdout,"   Commands: landscape       = ON\n");
         continue;
         }
      if (strcmp(argv[i],"-ps") == 0) {           /* postscript */
         postscript = OFF;
         fprintf(stdout,"   Commands: postscript      = OFF\n");
         continue;
         }
      if ( (strcmp(argv[i],"-print") == 0) ||
           (strcmp(argv[i],"-prt")   == 0) ) {    /* print */
         printplot = ON;
         fprintf(stdout,"   Commands: printplot       = ON\n");
         continue;
         }
      if ( (strcmp(argv[i],"-noplot") == 0) ||
           (strcmp(argv[i],"-nop")   == 0) ) {    /* noplot */
         noplot = ON;
         fprintf(stdout,"   Commands: noplot          = ON\n");
         continue;
         }
      if (strcmp(argv[i],"-s") == 0) {            /* scale factor */
         if (++i >= argc) usage(progname);
         scale = atof(argv[i]);
         if (scale < 0.1 || scale > 1) {
            scale = 1.0;
            fprintf(stdout,"   Warning : Invalid scale -");
            fprintf(stdout," scale reset to %f\n",scale);
         }
         fprintf(stdout,"   Commands: scale           = %f\n",scale);
         continue;
         }
      if (strcmp(argv[i],"-t") == 0) {            /* template file */
         if (++i >= argc) usage(progname);
         if ((tmpfile =  fopen(argv[i],"r")) == NULL) {
            fprintf(stdout,"   cat: can't open %s\n", template);
            fprintf(stdout,"        Default Plot types will be used\n");
         } else {
            fprintf(stdout,"   Commands: template-file   = \"%s\"\n",argv[i]);
            process_template(tmpfile);
            if (plottypes <=0) {
               plottypes = 17;
               fprintf(stdout,"   Warning : Read Error\n");
               fprintf(stdout,"      Default Plot types will be used\n");
            } else {
               line         = ON;
               marker       = ON;
               linechange   = ON;
               markerchange = ON;
            }
         }
         continue;
         }
      if (strcmp(argv[i],"-o") == 0) {            /* options file */
         if (++i >= argc) usage(progname);
         if ((optionfile =  fopen(argv[i],"r")) == NULL) {
           fprintf(stderr,"cat: can't open %s\n", argv[i]);
           usage(progname);
         } else {
            fprintf(stdout,"   Commands: options-file    = \"%s\"\n",argv[i]);
            parse(optionfile);
            fclose(optionfile);
         }
         continue;
         }
      if (argv[i][0] == '-' && argv[i][1] == 'P') {     /* Printer */
         strcpy(printer,argv[i]); 
         fprintf(stdout,"   Commands: Printer         = %s\n",printer);
         continue;
         }
      if (argv[i][0] == '-') usage(progname);
      if (argv[i][0] != '-' && argv[i][0] != '=') {
         FILEFOUND = TRUE;
         insert_pfile(argv[i]);
      }
   }
   if (FILEFOUND == FALSE) usage(progname);
   /* exit(0); */
}

/* read the plot data */
fileread(fp)
FILE *fp;
{
   nodeptr  insert_node();
   segmptr  insert_segm();
   float    rdfloat();
   double   conv_to_log();
   double   auto_round_down(), auto_round_up();

   extern   double  xmin, xmax, ymin, ymax;

   segmptr  S;
   nodeptr  N;
   double   x,y,z;
   int      ncurves, npts, i, m;

   xmin = rdfloat(fp);     
   xmax = rdfloat(fp);    
   ymin = rdfloat(fp);  
   ymax = rdfloat(fp);   
   if (xlog) {
      xmin = auto_round_down(xmin);
      xmax = auto_round_up  (xmax);
      xmin = conv_to_log(xmin);
      xmax = conv_to_log(xmax);
   }
   if (ylog) {
      ymin = auto_round_down(ymin);
      ymax = auto_round_up  (ymax);
      ymin = conv_to_log(ymin);
      ymax = conv_to_log(ymax);
   }

   ncurves = (int)rdfloat(fp);
   if (ncurves<=0) pltexit(1);
   for (m=0; m<ncurves; m++) {
      S = insert_segm();
      npts = (int)rdfloat(fp); 
      if (npts<=0) pltexit(2);
      for (i=0; i<npts; i++) {
         x = rdfloat(fp);  
         y = rdfloat(fp);   
         if (xlog) x = conv_to_log(x);
         if (ylog) y = conv_to_log(y);
         z = 0.0;
         N = insert_node(x,y,z,S);
      }
   }
}

double conv_to_log(x)
double x;
{
   if (x <= 0.0) {
      fprintf(stderr,"Error! Negative values (%f) \
are not acceptable for log plots!\n",x);
      exit(1);
   }
   return(log10(x));
}

float rdfloat(fp)  /* Read a float; skip the whole line if failed */
FILE *fp;
{
   int n;
   char c;
   float x;

   /* try to read a floating variable */
   /* if fail, then skip the whole line */
   while ( (n=fscanf(fp,"%f",&x)) != EOF && n!=1) {
      if (n!=1)
         while ( (c=getc(fp)) != EOF && c != '\n');
      }

   if (n==EOF) pltexit(0);

   return(x);
}

pltexit(mesg)
int mesg;
{
   switch (mesg) {
   case 0 : fprintf(stderr,"Error: Incorrect Format!\n");
           exit(1);
           break;
   case 1 : fprintf(stderr,"Error: Plot Curves <= 0\n");
           exit(1);
           break;
   case 2 : fprintf(stderr,"Error: Plot Points <= 0\n");
           exit(1);
           break;
   default: fprintf(stderr,"Error: Incorrect Format!\n");
           exit(1);
           break;
   }
}

usage (program)
char *program;
{
   fprintf(stderr,"usage: %s [-o options-file] [-Pprinter] [-s scale] \
[-ar] [-clip] [-e] \
[-g] [-l] [-ps] [-print] [-noplot] \
[-t template_file] [-xlog] [-ylog] plotfile1 plotfile2 ...\n",
program);
   exit(1);
}
