.TH DRAWPLOT L "23 August 1990"
.SH NAME
drawplot - plot program for Sunview, X-windows, HP2648s and Postscript
.SH SYNOPSIS
.B
drawplot
[-o \fIoptions-file\fR]
[-P\fIprinter\fR]
[-s \fIscale\fR] 
[-ar]
[-clip]
[-e]
[-g]
[-l]
[-noplot]
[-ps]
[-print]
[-xlog]
[-ylog]
[-t \fItemplate-file\fR]
plot-file
.SH DESCRIPTIONS
.PP
\fBDrawplot\fR is a program for drawing 2D plots on X10/X11 windows, SUNVIEW
displays, or HP2648 terminals. 
The program will also produce a POSTSCRIPT plot which
can be dumped out to an APPLE Laserwriter. 
\fBDrawplot\fR reads in x-y data from a \fIplot-file\fR and
manipulates that data according to options specified
either in the command-line or in a \fIoptions-file\fR.
The program then plots lines on a screen or dumps the plots 
to a POSTSCRIPT file.  
.PP
The \fIplot-file\fR input data
consists of alternating x and y values, in the format shown below.
.RS
.LP
.B
Data File Format (plot-file)
.LP
.nf
 xmin xmax ymin ymax 
 ncurves
 npts 
 z1
 z2
 ..
 ..
 ..
 npts 
 z1
 z2
 ..
 ..
 ..
.fi
.RE
.LP
In the above, \fIxmin\fR, \fIxmax\fR, \fIymin\fR and \fIymax\fR are
lower and upper bounds of the desired plot, \fIncurves\fR are the number of
curves to be plotted, and \fInpts\fR are the number of points in each
curve.  The data file can consist of more than one set of curves to be
plotted; each set (i.e. one set for each separate graph) is separated from
the next by a blank line.
.LP
Upon starting up the program, \fBdrawplot\fR will read in the data stored in
the \fIplot-file\fR, as well as any plotting options specified either in
the command line or in the \fIoptions-file\fR.
The plot will then be drawn on the screen if possible.
Finally, the user will be
prompted as to whether or not the POSTSCRIPT plot is to be sent to a printer.
.SH OPTIONS
.TP 5
.B \-o \fIoptions-file\fR
reads plotting options from the file \fIoptions-file\fR.
Each option specification consists of a keyword and its corresponding value.
The parser recognizes only a limited set of keywords; their values are
either numbers, quoted strings, or the words "on" and "off".
All the words in the option specification must be on the same line.
The pound sign (#) indicates that the remainder of the line is a comment to
be ignored by the parser.
.RS
.LP
.B
List of Options (options-file)
.RS
.LP
xlabel "LABEL" 	#[default = "X-AXIS"] 		- for the x-label   
.br
ylabel "LABEL" 	#[default = "Y-AXIS"] 		- for the y-label
.br
toplabel "LABEL" 	#[default = "X-Y LINE PLOT"] 	- for the top-label  
.br
autorange on/off 		#[default = on] 			- automatic axis ranging
.br
clip on/off 		#[default = off] 			- clips the picture
.br
equalscale on/off 	#[default = off] 			- for equal x-y scaling 
.br
grid on/off 		#[default = off] 			- draws a grid   
.br
noplot on/off 		#[default = off] 			- don't draw graphics plot 
.br
postscript on/off 		#[default = on] 			- for postscript (PS) plot 
.br
printplot on/off 		#[default = off] 			- send PS file to printer 
.br
printer "PRINTER"	#[default = $PRINTER] 		- define the printer
.br
line on/off 		#[default = on] 			- draw the line
.br
linechange on/off 	#[default = off] 			- change the linetypes
.br
marker on/off 		#[default = off] 			- draw the marker
.br
markerchange on/off 	#[default = off] 			- change the markertypes
.br
landscape on/off 		#[default = off] 			- print in landscape mode
.br
xlog on/off	 	#[default = off] 			- x-axis in log-scale
.br
ylog on/off	 	#[default = off] 			- y-axis in log-scale
.br
scale [0.1 - 1.0] 		#[default = 1.00] 		- scales the PS plot  
.br
xticks [1 - 20] 		#[default = 4] 			- no. of x-divisions  
.br
yticks [1 - 20] 		#[default = 4] 			- no. of y-divisions   
.br
sleeptime [1 - 20] 	#[default = 5] 			- SUNVIEW plot time   
.br
templatefile "template"	#[default = ""]			- plot/marker templates 
.RE
.RE
.TP 5
.B \-P\fIprinter\fR
specifies which printer to which to send the postscript plot.
The current default sets the printer name to the environment variable $PRINTER.
If this variable is not set, then the printer used is 
the lp550M printer in 550M Cory.
.TP 5
.B \-s \fIscale\fR
sets a scale factor.  This is used only for POSTSCRIPT plotting.
.TP 5
.B \-ar
forces no automatic ranging of the x and y axes.
Without this option, automatic ranging is set ON, and the program
tries to select the best axis-scales possible.  The autorange option
is also turned off when either \fIxticks\fR or \fIyticks\fR is defined
in the \fIoption-file\fR.
.TP 5
.B \-clip
forces clipping within the plot boundary. 
.TP 5
.B \-e 
causes the x and y grids to be drawn with equal scales.
.TP 5
.B \-g 
forces the grid to be drawn. 
.TP 5
.B \-l
prints the postscript plot in landscape mode (rotated).
.TP 5
.B \-ps
turns off the postscript plotting mode.
.TP 5
.B \-print
sends the postscript plot to the printer automatically.
.TP 5
.B \-noplot
stops the program from drawing on a SUN, X or HP graphics window.
Only the postscript plot will be made.
.TP 5
.B \-xlog
draws the x-axis on a log scale.
.TP 5
.B \-ylog
draws the y-axis on a log scale.
.TP 5
.B \-t \fItemplate-file\fR
reads the plot and marker template from the file \fItemplate-file\fR.
This is used to set the line type, marker type,
and line-label of each line.
The file should have 2 numbers per line; the first number corresponds to the
line type [0-10], while the second corresponds to the marker type [0-16].
The line-label is the first word that follows the
keywords "label", "linelabel" or "line_label".
The line-label should be placed inside quotes.
The number of lines in the \fItemplate-file\fR corresponds to the number of
different line or marker types that will appear in the plot - the curves
will cycle through these plot/marker types.
A sample \fItemplate-file\fR is shown below.
.RS
.LP
.B
Example Plot/Marker Template File (template-file)
.LP
.RS
.nf
pt_type = 1 		mkr_type = 0          label = "Line 1"
pt_type = 2 		mkr_type = 0          label = "Line 2"
pt_type = 3 		mkr_type = 1          label = "Line 3"
pt_type = 4 		mkr_type = 13         label = "Line 4"
pt_type = 5 		mkr_type = 2          label = "Line 5"
pt_type = 6 		mkr_type = 4          label = "Line 6"
.fi
.RE
.RE
.TP 5
.I host:display
opens a window on the given host and display
.TP 5
.B \-d \fIhost:display\fR
.TP 5
.B \=geom 
.TP 5
.B \-rv 
.TP 5
.B \-bw \fIborder-width\fR
.TP 5
.B \-bd \fIcolor\fR 
.TP 5
.B \-fg \fIcolor\fR 
.TP 5
.B \-bg \fIcolor\fR
.TP 5
.B \-fn \fIfont-name\fR
sets input options for the X-window system.
.SH AUTHOR
Kenny K.H. Toh (ktoh@mascot.berkeley.edu)
.SH FILES
\fIdataplot.ps\fR 	temporary POSTSCRIPT file
.SH SEE ALSO
contour(L), pdraw(L)
