/***    main.c   ***/

/* DRAWPLOT Version 1.6 : Kenny K.H. Toh, August 23, 1990 */

#include <stdio.h>
#include <strings.h>
#include "header.h"

main(argc,argv)
int argc;
char *argv[];
{
   FILE     *fopen(), *fp;
   extern   pfileptr pfilehead;
   pfileptr P;
   char     c;

   fprintf(stdout,"Drawplot V1.6  8/23/90\n");
   initialize_data();       /* Initialize the variables */
   fileopen(argc,argv);     /* read the data and options */

   /* process each plotfile */
   for (P=pfilehead; P!=NULL; P=P->next) {
      if ((fp = fopen(P->plotfile, "r")) == NULL) {
         fprintf(stderr,"cat: can't open %s\n", P->plotfile);
         exit(1);
      } 

      fileread(fp);            /* read the plotfile */
      draw_view(argc,argv);    /* draw the picture */
      clear_mem();             /* clear the memory */

      /* keep on reading until the next blank line */
      while ( (c=getc(fp)) != EOF && c!= '\n');
      while (c!=EOF) {
         if ( (c=getc(fp))!=EOF && c=='\n'){
            /* more curves coming up */
            fileread(fp);            /* read the plotfile */
            draw_view(argc,argv);    /* draw the picture */
            clear_mem();             /* clear the memory */
         } 
         /* read the rest of the line */
         while ( (c=getc(fp)) != EOF && c!= '\n');
      }
      fclose(fp);
   }
   fprintf(stdout,"Congratulations!!!  You have been TERMINATED...\n");
}

/* Initialize some data */
initialize_data()
{
   char            *getenv();
   char            *strcpy();
   char            *sprintf();
   extern char     *term;
   extern char     xlabel[MAXCHAR];
   extern char     ylabel[MAXCHAR];
   extern char     toplabel[MAXCHAR];
   extern char     printer[MAXCHAR];
   extern int      grid,equalscale,postscript,printplot,noplot,xlog,ylog,clip;
   extern int      xticks,yticks,sleeptime,landscape;
   extern int      line,linechange,marker,markerchange;
   extern int      linetype[MAXTYPE],markertype[MAXTYPE], plottypes;
   extern double   scale;
   extern segmptr  segmhead, segmtail;
   extern pfileptr pfilehead, pfiletail;

   int i;
   char *ptr;

   /* Terminal Type */
   term = getenv("TERM");
   fprintf(stdout,"   Terminal type             = %s\n",term);

   /* Labeling information */
   strcpy(xlabel , "X-AXIS");
   strcpy(ylabel , "Y-AXIS");
   strcpy(toplabel , "X-Y LINE PLOT");

   /* Plotting parameters/options */
   grid         = OFF;
   equalscale   = OFF;
   postscript   = ON;
   printplot    = OFF;
   noplot       = OFF;
   landscape    = OFF;
   xlog         = OFF;
   ylog         = OFF;
   clip         = OFF;
   line         = ON;
   linechange   = OFF;
   marker       = OFF; 
   markerchange = OFF;
   autorange    = ON;
   for (i=0; i<MAXTYPE; i++) linetype[i] = i+1;
   for (i=0; i<MAXTYPE; i++) markertype[i] = i;
   plottypes    = 17;
   scale        = 1.00;
   xticks       = 4;
   yticks       = 4;
   sleeptime    = 5;

   /* data */
   segmhead    = NULL;
   segmtail    = NULL;
   pfilehead   = NULL;
   pfiletail   = NULL;

   /* printer type */
   if ((ptr = getenv("PRINTER")) == NULL) 
      strcpy(printer,"-Plp550M");
   else 
      sprintf(printer,"-P%s",ptr);
   fprintf(stdout,"   Default Printer           = %s\n",printer);
}

draw_view(argc,argv)
int argc;
char *argv[];
{
   FILE   *fopen(), *ips;
   char   *sprintf();
   extern char *term;
   extern char printer[MAXCHAR];
   extern int postscript,printplot;
   char   c,command[MAXCHAR];

   if (!noplot) {
      /* SUN3 procedures */
      if (strcmp("sun",term)==0) {
         initsun(argc,argv);
         axessun();
         plotsun();
         endgrsun();
      }

      /* X procedures */
      if ( ((strcmp("xterm", term)==0) ||
            (strcmp("xterms",term)==0) || 
            (strcmp("vt200", term)==0) || 
            (strcmp("vt220", term)==0) || 
            (strcmp("vs100s",term)==0)) ) {
         InitWindow(argc,argv);
         RunOps();
      }

      /* hp2648 procedures */
      if ((strcmp("hp2648",term)==0) || (strcmp("2648",term)==0) ||
          (strcmp("hp2648a",term)==0)) {
         axeshp();
         plothp();
         endgrhp();
      }
   }

   /* Prepare for postscript output */
   if (postscript == ON) {
      if ((ips = fopen(PSFILE,"w")) == NULL) {
         fprintf(stderr,"cat: can't open %s\n",PSFILE);
         exit(1);
      }
      initps(ips);
      axesps(ips);
      plotps(ips);
      endgrps(ips);
   }

   /* Get postscript printout */
   if (postscript == ON) {
      if (printplot == OFF) {
      /* prompt for a print out */
         fprintf(stdout,"   Type return (or n/q) to ");
         fprintf(stdout,"send the plot to the %s printer :",printer);
         if ( (c=getc(stdin)) == '\n' || c == 'y' || c == 'Y' ) {
            sprintf(command,"%s %s %s","lpr -h",printer,PSFILE);
            system(command); 
            fprintf(stdout,"   Postscript file has been ");
            fprintf(stdout,"sent to the %s printer\n",printer);
         } else if (c == 'q')
            exit(1);
         if (c != '\n') while ( (c=getc(stdin)) != '\n');
      } else {
         /* print directly */
         sprintf(command,"%s %s %s","lpr -h",printer,PSFILE);
         system(command); 
         fprintf(stdout,"   Postscript file has been ");
         fprintf(stdout,"sent to the %s printer\n",printer);
      }
   }
}

/* clear the memory */
clear_mem()
{
   void delete_node();
   void delete_segm();

   segmptr S;
   extern  segmptr segmhead,segmtail;

   /* delete the nodes in each segment */
   for (S=segmhead; S!=NULL; S=S->next)
      while (S->head!=NULL) delete_node(S->tail,S);

   /* now delete all the segments */
   while (segmhead!=NULL) delete_segm(segmtail);

   segmhead = NULL;
   segmtail = NULL;
}

/* count the number of curves and the number of points per curve */
count_curves()
{
   extern  segmptr segmhead;
   segmptr S;
   nodeptr N;
   int     npts,ncurvs=0;

   for (S=segmhead; S!=NULL; S=S->next) ncurvs++;

   for (S=segmhead; S!=NULL; S=S->next) {
      npts=0;
      for (N=S->head; N!=NULL; N=N->next) npts++;
      fprintf(stdout,"   Num of Curves= %d  Num of points= %d\n",ncurvs,npts);
   }
}
