/*   plotX.c - all the X11 routines  ***/

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/keysym.h>
#include <stdio.h>
#include <strings.h>
#include <math.h>

#include "header.h" 
#include "Xdefs.h"
#include "iconX11.ic"

/* colors */
#define DEFAULT_BORDER_COLOR  "Blue"
#define DEFAULT_BACK_COLOR    "MidNightBlue"
#define DEFAULT_FORE_COLOR    "White"
#define DEFAULT_MOUSE_COLOR   "White"
#define DEFAULT_ICON_COLOR    "Red"
#define BDR_C                 0
#define BACK_C                1
#define FORE_C                2
#define MOUSE_C               3
#define ICON_C                4
#define LINE1_C               5
#define LINE2_C               6
#define LINE3_C               7
#define LINE4_C               8
#define LINE5_C               9
#define MAX_COLORS            10

unsigned long     border_pixel;                   /* color of border */
unsigned long     background_pixel;               /* color of background */
unsigned long     foreground_pixel;               /* color of foreground */
unsigned long     mouseground_pixel;              /* mouse cursor color */
unsigned long     icon_pixel;                     /* icon color */
unsigned long     colors[MAX_COLORS];             /* color pixel values */
char              *clr_name[MAX_COLORS];          /* color names */

/* line-types */
#  define DOTTED       2
#  define SHORT_DOTTED 2
#  define LONG_DOTTED  2
#  define DASHED       2
#  define DOT_DASHED   4
#  define DASH_DOTTED  4
#  define SHORT_DASHED 2
#  define LONG_DASHED  2
#  define EQUAL_DASHED 2
#  define ODD_DASHED   3
   static unsigned char dash_list_dotted[DOTTED]             = {1,6};
   static unsigned char dash_list_short_dotted[SHORT_DOTTED] = {1,3};
   static unsigned char dash_list_long_dotted[LONG_DOTTED]   = {1,8};
   static unsigned char dash_list_dashed[DASHED]             = {4,4};
   static unsigned char dash_list_dot_dashed[DOT_DASHED]     = {3,4,3,1};
   static unsigned char dash_list_dash_dotted[DASH_DOTTED]   = {4,3,1,3};
   static unsigned char dash_list_short_dashed[SHORT_DASHED] = {3,5};
   static unsigned char dash_list_equal_dashed[EQUAL_DASHED] = {6,6};
   static unsigned char dash_list_long_dashed[LONG_DASHED]   = {7,4};
   static unsigned char dash_list_odd_dashed[ODD_DASHED]     = {1,2,3};

/* plot information */
#define X_ORG            5
#define Y_ORG            5
#define X_DIM            700
#define Y_DIM            500
#define DEFAULT_BDR_DIM  100

#define X_MKR            4
#define X_LIN            11

int Xxmin, Xymin, Xxmax, Xymax;

/* Misc */
#define DEFAULT_BORDER_WIDTH  3
#define DEFAULT_FONT          "8x13"

short gray_bits[16] = {
   0xaaaa, 0x5555, 0xaaaa, 0x5555,
   0xaaaa, 0x5555, 0xaaaa, 0x5555,
   0xaaaa, 0x5555, 0xaaaa, 0x5555,
   0xaaaa, 0x5555, 0xaaaa, 0x5555};

Display     *display;                   /* display */
int         screen;                     /* screen */
Window      window;                     /* parent window */
XFontStruct *font_info;                 /* font information */
XSizeHints  size_hints;                 /* window info for manager */
GC          gc;                         /* graphics context */

InitWindow(argc,argv)
int  argc;
char **argv;
{
   Pixmap     icon_pixmap;
   XGCValues  gcvalues;
   char       *sprintf(), default_geometry[100];
   int        x, y, width, height;

   /* Open display first, and get screen info */
   if (!(display = XOpenDisplay(display_name))) {
      fprintf(stderr,"%s:Could not open display %s\n",
              progname,display_name);
      exit(1);
   }
   screen = DefaultScreen(display);
   sprintf(default_geometry,"=%dx%d+%d+%d",X_DIM,Y_DIM,X_ORG,Y_ORG);

   /* Merge the options first */
   Merge_Options();

   /* allocate the colors */
   Alloc_Colors();

   /* Initialize the window */
   if (!XGeometry(display,screen,geometry,default_geometry,border_width,
                 0,0,0,0,&x,&y,&width,&height)) {
      x      = X_ORG;
      y      = Y_ORG;
      width  = X_DIM;
      height = Y_DIM;
   }
   window = XCreateSimpleWindow(display, RootWindow(display,screen),
                                x, y, width, height, border_width,
                                border_pixel, background_pixel);

   /* Initialize size hint property for window manager */
   size_hints.flags      = PPosition | PSize | PMinSize;
   size_hints.x          = x;
   size_hints.y          = y;
   size_hints.width      = width;
   size_hints.height     = height;
   size_hints.min_width  = X_DIM;
   size_hints.min_height = Y_DIM;

   /* Create pixmap of depth 1 (bitmap) for icon */
   icon_pixmap = XCreateBitmapFromData(display, window, icon_bitmap_bits,
                        icon_bitmap_width, icon_bitmap_height);

   /* set properties for window manager */
   XSetStandardProperties(display,window, "Drawplot Program", "Drawplot",
                          icon_pixmap, argv, argc, &size_hints);

   /* Select event types wanted */
   XSelectInput(display,window,ExposureMask | ButtonPressMask  |
                KeyPressMask | StructureNotifyMask );

   /* Create GC for text and drawing */
   gc = XCreateGC(display, window, 0, &gcvalues);

   /* Find a font and load it */
   if (!(font_info = XLoadQueryFont(display,font_name))){
      fprintf(stderr,"%s:Could not open font %s\n",progname,font_name);
      exit(1);
   }
   XSetFont(display, gc, font_info->fid);

   /* specify foreground */
   XSetForeground(display, gc, foreground_pixel);

   /* Display the window */
   XMapWindow(display, window);
}

/* Initialize and merge the various options */
Merge_Options()
{
   char    *option;

   /* process the various options */
   if (reverse < 0) {
      if ((option = XGetDefault(display,progname,"ReverseVideo")) != NULL)
         if (strcmp(option,"on")) reverse = 1;
   }
   if (!geometry) {
      option = XGetDefault(display,progname,"Geometry");
      geometry = option ? option : "=+5+5";
   }
   if (!font_name) {
      /* option = XGetDefault(display,progname,"BodyFont"); */
      font_name = DEFAULT_FONT;
   }
   if (!border_color) {
      option = XGetDefault(display,progname,"Border");
      border_color = option ? option : DEFAULT_BORDER_COLOR;
   }
   if (!back_color) {
      option = XGetDefault(display,progname,"Background");
      back_color = option ? option : DEFAULT_BACK_COLOR;
   }
   if (!fore_color) {
      option = XGetDefault(display,progname,"Foreground");
      fore_color = option ? option : DEFAULT_FORE_COLOR;
   }
   if (!mouse_color) {
      option = XGetDefault(display,progname,"Mouse");
      mouse_color = option ? option : DEFAULT_MOUSE_COLOR;
   }
   if (!icon_color) {
      option = XGetDefault(display,progname,"Icon");
      icon_color = option ? option : DEFAULT_ICON_COLOR;
   }
   if (border_width <= 0) {
      option = XGetDefault(display,progname,"BorderWidth");
      border_width = option ? atoi(option) : DEFAULT_BORDER_WIDTH;
   }
}

/* allocate the colors on color display */
Alloc_Colors()
{
   XColor        cdef;
   Colormap      cmap;
   int           i, depth;
   unsigned long tmp_pixel;

   /* get info on depth and colormap */
   depth = DisplayPlanes(display,screen);
   cmap  = DefaultColormap(display,screen);

   /* associate named colors with pixel values */
   if (depth == 1) {       /* one-plane monochrome */
      colors[BDR_C]     = BlackPixel(display,screen);
      colors[BACK_C]    = BlackPixel(display,screen);
      colors[FORE_C]    = WhitePixel(display,screen);
      colors[MOUSE_C]   = colors[FORE_C];            
      colors[ICON_C]    = colors[FORE_C];            
      for (i=LINE1_C; i<=LINE5_C; i++) colors[i] = colors[FORE_C];
   } else {                /* color */
      clr_name[BDR_C]   = border_color;
      clr_name[BACK_C]  = back_color;
      clr_name[FORE_C]  = fore_color;
      clr_name[MOUSE_C] = mouse_color;
      clr_name[ICON_C]  = icon_color;
      clr_name[LINE1_C] = "yellow";
      clr_name[LINE2_C] = "cyan";
      clr_name[LINE3_C] = "green";
      clr_name[LINE4_C] = "red";
      clr_name[LINE5_C] = "blue";
      for (i=0; i<MAX_COLORS; i++) 
         if (XAllocNamedColor(display,cmap,clr_name[i],&cdef,&cdef))
            colors[i] = cdef.pixel;
         else {
            fprintf(stderr,"%s: Can't allocate color %s\n",
                    progname, clr_name[i]);
            exit(1);
         } /* end if */
   }  /* end if */

   /* assign pixel values */
   border_pixel      = colors[BDR_C];
   background_pixel  = colors[BACK_C];
   foreground_pixel  = colors[FORE_C];
   mouseground_pixel = colors[MOUSE_C];
   icon_pixel        = colors[ICON_C];

   /* reverse video */
   if (reverse) {
      tmp_pixel        = background_pixel;
      background_pixel = foreground_pixel;
      foreground_pixel = tmp_pixel;
      if (border_pixel == background_pixel)
         border_pixel = foreground_pixel;
      if (mouseground_pixel == background_pixel) 
         mouseground_pixel = foreground_pixel;
      for (i=LINE1_C; i<=LINE5_C; i++)
         if (colors[i]==background_pixel) colors[i] = foreground_pixel;
   }
}

RunOps()
{
#  define   SMALL 1
#  define   OK    0
   XEvent   event;
   KeySym   keysym;
   XComposeStatus compose;
   char     buffer[1];
   int      bufsize = 1;
   int      window_size = OK;
   int      Width, Height;       

   /* Initialize window size */
   Width  = X_DIM;
   Height = Y_DIM;

   /* event loop */
   while(1) {
      XNextEvent(display, &event);
      switch (event.type) {
      case Expose:
         /* get rid of all other Expose events on the queue */
         while (XCheckTypedEvent(display, Expose, &event));
         /* window was resized too small to use */
         if (window_size == SMALL) {
            Width  = X_DIM;
            Height = Y_DIM;
         }
         ShowView(Width, Height);
         break;
      case ConfigureNotify:
         /* window has been resized */
         /* change width and height */
         Width  = event.xconfigure.width;
         Height = event.xconfigure.height;
         if ((Width < size_hints.min_width) ||
             (Height < size_hints.min_height) ) 
            window_size = SMALL;
         else 
            window_size = OK;
         break;
      case KeyPress:
         XLookupString(&event,buffer,bufsize,&keysym,&compose);
         if ((keysym == XK_q) || (keysym == XK_Q)) {
            XUnloadFont(display, font_info->fid);
            XFreeGC(display, gc);
            XCloseDisplay(display);
            return;
         }
         break;
      case ButtonPress:
         XUnloadFont(display, font_info->fid);
         XFreeGC(display, gc);
         XCloseDisplay(display);
         return;
         break;
      default:
         break;
      } /* end switch */
   } /* end while */
}

ShowView(Width, Height)
int      Width, Height;       
{
   axesX(Width, Height);
   plotX();
}

/* draw the axes */
axesX(Width, Height)
int   Width, Height;       
{
   char           *sprintf();
   extern char    xlabel[MAXCHAR];
   extern char    ylabel[MAXCHAR];
   extern char    toplabel[MAXCHAR];
   extern int     autorange, equalscale, clip, linechange, markerchange;
   extern double  xmin, xmax, ymin, ymax;

   XRectangle     rectangles[1];
   int    Xwid, Ywid;
   double ratio;
   char   text[100], c, time[MAXCHAR];
   int    text_len, text_width, text_xpos, text_ypos;
   int    font_height, font_width, label_len;
   int    i;

   /* Initialize */
   Xxmin = DEFAULT_BDR_DIM;
   Xymin = DEFAULT_BDR_DIM;
   Xwid  = Width  - 2*DEFAULT_BDR_DIM;
   if (linechange || markerchange)
      Xwid  = Width  - 3*DEFAULT_BDR_DIM;
   Ywid  = Height - 2*DEFAULT_BDR_DIM;
   Xxmax = Xxmin + Xwid;
   Xymax = Xymin + Ywid;

   if (equalscale == ON) {
      /* max Xwid = 400, max Ywid = 300 */
      ratio = (ymax-ymin)/(xmax-xmin);
      if (ratio < ((double)Ywid)/((double)Xwid))
         Ywid  = ratio*Xwid;
      else
         Xwid  = Ywid/ratio;
      Xxmax = Xxmin + Xwid;
      Xymax = Xymin + Ywid;
   }

   /* Draw the main axes */
   XSetLineAttributes(display,gc,1,LineSolid,CapButt,JoinBevel);
   XDrawRectangle(display,window,gc,Xxmin,Xymin,Xwid,Ywid);

   /* Labels */
   font_height = font_info->max_bounds.ascent +
                 font_info->max_bounds.descent;
   font_width  = font_info->max_bounds.rbearing -
                 font_info->max_bounds.lbearing;

   /* Top Label */
   text_len   = strlen(toplabel);
   text_width = XTextWidth(font_info,toplabel,text_len);
   text_xpos  = (Xxmin + Xxmax - text_width)/2;
   text_ypos  = Xymin - 3*font_height;
   XDrawString(display,window,gc,text_xpos,text_ypos,toplabel,text_len);

   /* Draw the tick marks and labels */
   label_len = 6;
   if (!autorange) {
      plot_Xticks(&label_len);
   } else {
      plot_XAticks(&label_len);
   }

   /* X-Axis Label */
   text_len   = strlen(xlabel);
   text_width = XTextWidth(font_info,xlabel,text_len);
   text_xpos  = (Xxmin + Xxmax - text_width)/2;
   text_ypos  = Xymax + 4*font_height;
   XDrawString(display,window,gc,text_xpos,text_ypos,xlabel,text_len);

   /* Y-Axis Label */
   /* Don't know how to rotate, so print character by character */
   text_len   = strlen(ylabel);
   text_xpos  = Xxmin - (label_len+6)*font_width;
   text_ypos  = (Xymin + Xymax)/2 + 0.5*font_height 
                - 0.50*(text_len-1)*((int)(1.2*font_height));
   for (i=0; (c=ylabel[i]) != '\0'; i++) {
      text[0] = c;
      XDrawString(display,window,gc,text_xpos,text_ypos,text,1);
      text_ypos += (int)(1.2*font_height);
   }

   /* plot linetypes and markers against labels */
   if (linechange || markerchange) plot_labels();

   /* put on a time label */
   get_localtime(time);
   text_len   = strlen(time);
   text_width = XTextWidth(font_info,time,text_len);
   text_xpos  = Xxmin ;
   text_ypos  = Xymin - 6*font_height;
   XDrawString(display,window,gc,text_xpos,text_ypos,time,text_len);

   /* clip henceforth */
   if (clip) {
      rectangles[0].x      = Xxmin;
      rectangles[0].y      = Xymin;
      rectangles[0].width  = Xwid;
      rectangles[0].height = Ywid;
      XSetClipRectangles(display,gc,0,0,rectangles,1,Unsorted);
   }
}

/* Draw the tick marks and labels */
plot_Xticks(label_len)
int  *label_len;
{
   char           *sprintf();
   extern int     grid,xticks,yticks,xlog,ylog;
   extern double  xmin, xmax, ymin, ymax;

   double xdx, ydy, xintv, yintv;
   double vallbl;
   int    Xxtmp, Xytmp;
   int    font_height, font_width;
   int    i, llen=0;

   /* Labels */
   font_height = font_info->max_bounds.ascent +
                 font_info->max_bounds.descent;
   font_width  = font_info->max_bounds.rbearing -
                 font_info->max_bounds.lbearing;

   /* useful data */
   xintv = (Xxmax-Xxmin)/xticks;
   yintv = (Xymax-Xymin)/yticks;
   xdx   = (xmax-xmin)/xticks;
   ydy   = (ymax-ymin)/yticks;

   /* reset the number of ticks */
   if (xlog) {
      xticks = (int)(xmax - xmin - 1e-5) + 1;
      xmax   = xmin + xticks;
   } 
   if (ylog) {
      yticks = (int)(ymax - ymin - 1e-5) + 1;
      ymax   = ymin + yticks;
   } 

   /* xticks */
   for (i=0; i<=xticks; i++) {
      Xxtmp = Xxmin + (int)(i*xintv);
      XDrawLine(display,window,gc,Xxtmp,Xymin,Xxtmp,Xymin+10);
      XDrawLine(display,window,gc,Xxtmp,Xymax,Xxtmp,Xymax-10);
      vallbl = xmin +i*xdx;
      plot_Xxticklabel(vallbl,Xxtmp,font_height,font_width);
   }

   /* yticks */
   for (i=0; i<=yticks; i++) {
      Xytmp = Xymin + (int)(i*yintv);
      XDrawLine(display,window,gc,Xxmin,Xytmp,Xxmin+10,Xytmp);
      XDrawLine(display,window,gc,Xxmax,Xytmp,Xxmax-10,Xytmp);
      vallbl = ymax -i*ydy;
      plot_Xyticklabel(vallbl,Xytmp,font_height,font_width,&llen);
   }

   /* return the label length */
   if (llen < *label_len) *label_len = llen;

   /* Work on the grid */
   if (grid == ON) {
      XSetLineAttributes(display,gc,0,LineOnOffDash,CapButt,JoinBevel);
      XSetDashes(display,gc,0,dash_list_long_dotted,LONG_DOTTED);
      for (i=1; i<xticks; i++) {
         Xxtmp = Xxmin + (int)(i*xintv);
         XDrawLine(display,window,gc,Xxtmp,Xymin+10,Xxtmp,Xymax-10);
      }
      for (i=1; i<yticks; i++) {
         Xytmp = Xymin + (int)(i*yintv);
         XDrawLine(display,window,gc,Xxmin+10,Xytmp,Xxmax-10,Xytmp);
      }
      XSetLineAttributes(display,gc,0,LineSolid,CapButt,JoinBevel);
   }
}

/* Draw the tick marks and labels with automatic ranging */
plot_XAticks(label_len)
int  *label_len;
{
   char           *sprintf();
   double         log10();
   extern int     grid,xticks,yticks,xlog,ylog;
   extern double  xmin, xmax, ymin, ymax;

   double xmin2, xmax2, deltx, ymin2, ymax2, delty;
   double xtmp, ytmp, xt, yt, xfactor, yfactor;
   int    Xxtmp, Xytmp;
   int    font_height, font_width;
   int    i, nticks, llen=0;

   /* Labels */
   font_height = font_info->max_bounds.ascent +
                 font_info->max_bounds.descent;
   font_width  = font_info->max_bounds.rbearing -
                 font_info->max_bounds.lbearing;

   /* 
    * useful data 
    * X = Xxmin + (x-xmin)*(Xxmax-Xxmin)/(xmax-xmin) 
    * Y = Xymax + (y-ymin)*(Xymin-Xymax)/(ymax-ymin) 
    */
   xfactor = (Xxmax - Xxmin)/(xmax-xmin);
   yfactor = (Xymin - Xymax)/(ymax-ymin);
   
   /* get rounded limits */
   get_autorange(xmin,xmax,ymin,ymax,
                 &xmin2,&xmax2,&deltx,&ymin2,&ymax2,&delty);

   /* xticks */

   /* draw minor tick marks */
   xtmp = xmin2;
   while (xtmp < xmax) {
      for (i=1; i<10; i++) {
         xt = xtmp + i*0.1*deltx;
         if (xlog) xt = xtmp + log10((double)i); 
         if ((xt > xmin) && (xt < xmax)) {
            Xxtmp = Xxmin + (xt-xmin)*xfactor;
            if (i==5) {
               XDrawLine(display,window,gc,Xxtmp,Xymin,Xxtmp,Xymin+7);
               XDrawLine(display,window,gc,Xxtmp,Xymax,Xxtmp,Xymax-7);
            } else {
               XDrawLine(display,window,gc,Xxtmp,Xymin,Xxtmp,Xymin+4);
               XDrawLine(display,window,gc,Xxtmp,Xymax,Xxtmp,Xymax-4);
            }
         }
      }
      xtmp += deltx;
   }

   /* major tickmarks and accompanying labels */
   xtmp = xmin2 + deltx;
   nticks = 0;
   while (xtmp < xmax-0.0001) { 
      Xxtmp = Xxmin + (xtmp-xmin)*xfactor;
      XDrawLine(display,window,gc,Xxtmp,Xymin,Xxtmp,Xymin+10);
      XDrawLine(display,window,gc,Xxtmp,Xymax,Xxtmp,Xymax-10);
      plot_Xxticklabel(xtmp,Xxtmp,font_height,font_width);
      xtmp += deltx;
      nticks ++;
   }

   /* put labels on the min and max points */
   if ((nticks==0) || ((xmin - xmin2) < 0.5*deltx)) {
      plot_Xxticklabel(xmin,Xxmin,font_height,font_width);
   }

   /* put labels on the min and max points */
   if ((nticks==0) || ((xtmp - xmax) < 0.5*deltx)) {
      plot_Xxticklabel(xmax,Xxmax,font_height,font_width);
   }

   /* yticks */

   /* draw minor tick marks */
   ytmp = ymin2;
   while (ytmp < ymax) {
      for (i=1; i<10; i++) {
         yt = ytmp + i*0.1*delty;
         if (ylog) yt = ytmp + log10((double)i); 
         if ((yt > ymin) && (yt < ymax)) {
            Xytmp = Xymax + (yt-ymin)*yfactor;
            if (i==5) {
               XDrawLine(display,window,gc,Xxmin,Xytmp,Xxmin+7,Xytmp);
               XDrawLine(display,window,gc,Xxmax,Xytmp,Xxmax-7,Xytmp);
            } else {
               XDrawLine(display,window,gc,Xxmin,Xytmp,Xxmin+4,Xytmp);
               XDrawLine(display,window,gc,Xxmax,Xytmp,Xxmax-4,Xytmp);
            }
         }
      }
      ytmp += delty;
   }

   /* major tickmarks and accompanying labels */
   ytmp = ymin2 + delty;
   nticks = 0;
   while (ytmp < ymax-0.0001) { 
      Xytmp = Xymax + (ytmp-ymin)*yfactor;
      XDrawLine(display,window,gc,Xxmin,Xytmp,Xxmin+10,Xytmp);
      XDrawLine(display,window,gc,Xxmax,Xytmp,Xxmax-10,Xytmp);
      plot_Xyticklabel(ytmp,Xytmp,font_height,font_width,&llen);
      ytmp += delty;
      nticks ++;
   }

   /* put labels on the min and max points */
   if ((nticks==0) || ((ymin - ymin2) < 0.5*delty)) {
      plot_Xyticklabel(ymin,Xymax,font_height,font_width,&llen);
   }

   /* put labels on the min and max points */
   if ((nticks==0) || ((ytmp - ymax) < 0.5*deltx)) {
      plot_Xyticklabel(ymax,Xymin,font_height,font_width,&llen);
   }

   /* return the label length */
   if (llen < *label_len) *label_len = llen;

   /* Work on the grid */
   if (grid == ON) {
      XSetLineAttributes(display,gc,0,LineOnOffDash,CapButt,JoinBevel);
      XSetDashes(display,gc,0,dash_list_long_dotted,LONG_DOTTED);
      xtmp = xmin2 + deltx;
      while (xtmp < xmax) { 
         Xxtmp = Xxmin + (xtmp-xmin)*xfactor;
         XDrawLine(display,window,gc,Xxtmp,Xymin+10,Xxtmp,Xymax-10);
         xtmp += deltx;
      }
      ytmp = ymin2 + delty;
      while (ytmp < ymax) { 
         Xytmp = Xymax + (ytmp-ymin)*yfactor;
         XDrawLine(display,window,gc,Xxmin+10,Xytmp,Xxmax-10,Xytmp);
         ytmp += delty;
      }
      XSetLineAttributes(display,gc,0,LineSolid,CapButt,JoinBevel);
   }
}

/* plot the tick label on the x-axis */
plot_Xxticklabel(vallbl,Xxtmp,font_height,font_width)
double vallbl;
int    Xxtmp, font_height, font_width;
{
   char   *sprintf();
   extern int xlog;
   int    text_len, text_width, text_xpos, text_ypos;
   char   text[MAXCHAR];

   if (xlog) vallbl = pow(10.0,vallbl);
   sprintf(text,"%.3g",vallbl);
   text_len   = strlen(text);
   text_width = XTextWidth(font_info,text,text_len);
   text_xpos  = Xxtmp  - text_width/2;
   text_ypos  = Xymax + 2*font_height;
   XDrawString(display,window,gc,text_xpos,text_ypos,text,text_len);
}

/* plot the tick label on the y-axis */
plot_Xyticklabel(vallbl,Xytmp,font_height,font_width,llen)
double vallbl;
int    Xytmp, font_height, font_width, *llen;
{
   char   *sprintf();
   extern int ylog;
   int    text_len, text_width, text_xpos, text_ypos;
   char   text[MAXCHAR];

   if (ylog) vallbl = pow(10.0,vallbl);
   sprintf(text,"%.3g",vallbl);
   text_len   = strlen(text);
   if (text_len > *llen) *llen = text_len;
   text_width = XTextWidth(font_info,text,text_len);
   text_xpos  = Xxmin - text_width - font_width;
   text_ypos  = Xytmp  + font_height/2;
   XDrawString(display,window,gc,text_xpos,text_ypos,text,text_len);
}

/* draw the plot */
plotX()
{
   extern int     line, linechange, marker, markerchange;
   extern int     linetyp[MAXTYPE],markertype[MAXTYPE], plottypes;
   extern double  xmin, xmax, ymin, ymax;
   extern segmptr segmhead;

   segmptr S;
   nodeptr N;
   double  idx,jdy;
   int     j,npts,nplt=0;
   int     kl,km;
   XPoint  points[MAXPTS];

   idx = (Xxmax-Xxmin)/(xmax-xmin);
   jdy = (Xymax-Xymin)/(ymin-ymax);  /* because of strange x format */

   kl = linetype[nplt];
   km = markertype[nplt];

   S = segmhead;
   for ( ; S!=NULL; S=S->next) {
      N = S->head;
      j = 0;
      /* rescale points, save in array */
      for ( ; N!=NULL && j<MAXPTS; N=N->next) {
         points[j].x = (int)((N->x-xmin)*idx) + Xxmin;
         points[j].y = (int)((N->y-ymax)*jdy) + Xymin;
         j++;
      }
      npts = j;

      /* plot the points */
      plot_line(points,npts,kl);

      /* plot the markers */
      plot_markers(points,npts,kl,km);

      /* increment the plot-type by increasing the array index */
      nplt++;
      if (nplt >= plottypes) nplt = nplt % plottypes;
      /* now look up the line and marker type */
      if (linechange   == ON) kl = linetype[nplt];  
      if (markerchange == ON) km = markertype[nplt];  
   }
}

/* plot the points */
plot_line(points,npts,kl)
XPoint points[];
int    npts, kl;
{
   int linetypX();
   extern int line;
   int linepattern;

   if (line == ON && kl!=0) {
      linecolX(kl);
      linepattern = linetypX(kl);
      if (linepattern)
         XDrawLines(display,window,gc,points,npts,CoordModeOrigin);
      else      /* point_plot */
         XDrawPoints(display,window,gc,points,npts,CoordModeOrigin);
      linecolX(0);
   }
}

/* plot the markers */
plot_markers(points,npts,kl,km)
XPoint points[];
int    npts, kl,km;
{
   extern int marker;
   int    j;

   if (marker == ON || (kl==0 && km==0)) {
      XSetLineAttributes(display,gc,0,LineSolid,CapButt,JoinBevel);
      km = km % X_MKR;
      switch (km) {
      case 0 : 
         XDrawPoints(display,window,gc,points,npts,CoordModeOrigin);
         break;
      case 1 : 
         for (j=0; j < npts; j++) 
         XDrawRectangle(display,window,gc,points[j].x-1,points[j].y-1,2,2);
         break;
      case 2 : 
         for (j=0; j < npts; j++) {
         XDrawLine(display,window,gc,points[j].x-2,points[j].y-2,
                                     points[j].x+2,points[j].y+2);
         XDrawLine(display,window,gc,points[j].x+2,points[j].y-2,
                                     points[j].x-2,points[j].y+2);
         }
         break;
      case 3 : 
         for (j=0; j < npts; j++) 
         XDrawRectangle(display,window,gc,points[j].x-2,points[j].y-2,4,4);
         break;
      }
   }
}

/* print the labels */
plot_labels()
{
   extern  int line, linechange, marker, markerchange;
   extern  int linetyp[MAXTYPE], markertype[MAXTYPE], plottypes;
   extern  char linelabel[MAXTYPE][MAXCHAR];
   segmptr S;
   char    text[MAXCHAR];
   int     ncurves, nlines;
   int     i,kl, km, npts=0, nplt=0;
   int     text_len, text_width, text_xpos, text_ypos;
   int     font_height, font_width;
   XPoint  points[MAXPTS];

   /* count no of curves */
   ncurves = 0;
   for (S=segmhead; S!=NULL; S=S->next) ncurves++;
 
   /* no of lines to be plotted */
   nlines = (plottypes < ncurves) ? plottypes : ncurves;

   /* set the points */
   points[0].x = Xxmax + 20;
   points[0].y = Xymin + 20;
   points[1].x = Xxmax + 60;
   points[1].y = Xymin + 20;
   npts        = 2;

   /* set font info */
   font_height = font_info->max_bounds.ascent +
                 font_info->max_bounds.descent;
   font_width  = font_info->max_bounds.rbearing -
                 font_info->max_bounds.lbearing;

   for (i=0; i<nlines; i++) {
      /* set linetype and markertype */
      kl = linetype[nplt];
      km = markertype[nplt];
     
      /* draw line */
      plot_line(points,npts,kl);

      /* draw marker */
      plot_markers(points,npts,kl,km);

      /* draw linelabel */
      strcpy(text,linelabel[nplt]);
      text_len   = strlen(text);
      text_width = XTextWidth(font_info,text,text_len);
      text_xpos  = points[1].x + 20; 
      text_ypos  = points[1].y + font_height/2;
      XDrawString(display,window,gc,text_xpos,text_ypos,text,text_len);

      /* increment the plot-type by increasing the array index */
      nplt++;
      if (nplt >= plottypes) nplt = nplt % plottypes;
      /* now look up the line and marker type */
      if (linechange   == ON) kl = linetype[nplt];  
      if (markerchange == ON) km = markertype[nplt];  

      /* change ypos */
      points[0].y += font_height * 3/2;
      points[1].y += font_height * 3/2;
   }
}

/* set the line type */
linetypX(linetyp)
int linetyp;
{
   int      linepat = ON;
   linetyp = linetyp % X_LIN;
   switch (linetyp) {
   case 0 : XSetLineAttributes(display,gc,0,LineSolid,CapButt,JoinBevel);
            linepat = OFF;
            break;
   case 1 : XSetLineAttributes(display,gc,0,LineSolid,CapButt,JoinBevel);
            break;
   case 2 : XSetLineAttributes(display,gc,0,LineOnOffDash,CapButt,JoinBevel);
            XSetDashes(display,gc,0,dash_list_dashed,DASHED);
            break;
   case 3 : XSetLineAttributes(display,gc,0,LineOnOffDash,CapButt,JoinBevel);
            XSetDashes(display,gc,0,dash_list_dotted,DOTTED);
            break;
   case 4 : XSetLineAttributes(display,gc,0,LineOnOffDash,CapButt,JoinBevel);
            XSetDashes(display,gc,0,dash_list_dot_dashed,DOT_DASHED);
            break;
   case 5 : XSetLineAttributes(display,gc,0,LineOnOffDash,CapButt,JoinBevel);
            XSetDashes(display,gc,0,dash_list_short_dotted,SHORT_DOTTED);
            break;
   case 6 : XSetLineAttributes(display,gc,0,LineOnOffDash,CapButt,JoinBevel);
            XSetDashes(display,gc,0,dash_list_short_dashed,SHORT_DASHED);
            break;
   case 7 : XSetLineAttributes(display,gc,0,LineOnOffDash,CapButt,JoinBevel);
            XSetDashes(display,gc,0,dash_list_odd_dashed,ODD_DASHED);
            break;
   case 8 : XSetLineAttributes(display,gc,0,LineOnOffDash,CapButt,JoinBevel);
            XSetDashes(display,gc,0,dash_list_long_dashed,LONG_DASHED);
            break;
   case 9 : XSetLineAttributes(display,gc,0,LineOnOffDash,CapButt,JoinBevel);
            XSetDashes(display,gc,0,dash_list_dash_dotted,DASH_DOTTED);
            break;
   case 10: XSetLineAttributes(display,gc,0,LineOnOffDash,CapButt,JoinBevel);
            XSetDashes(display,gc,0,dash_list_equal_dashed,EQUAL_DASHED);
            break;
   }
   return(linepat);
}

/* choose a linecolor */
linecolX(linetyp)
int linetyp;
{
   linetyp = linetyp % X_LIN;
   switch (linetyp) {
      case 0 : XSetForeground(display,gc,foreground_pixel); break;
      case 1 : XSetForeground(display,gc,colors[LINE1_C]); break;
      case 2 : XSetForeground(display,gc,colors[LINE2_C]); break;
      case 3 : XSetForeground(display,gc,colors[LINE3_C]); break;
      case 4 : XSetForeground(display,gc,colors[LINE4_C]); break;
      case 5 : XSetForeground(display,gc,colors[LINE5_C]); break;
      case 6 : XSetForeground(display,gc,colors[LINE1_C]); break;
      case 7 : XSetForeground(display,gc,colors[LINE2_C]); break;
      case 8 : XSetForeground(display,gc,colors[LINE3_C]); break;
      case 9 : XSetForeground(display,gc,colors[LINE4_C]); break;
      case 10: XSetForeground(display,gc,colors[LINE5_C]); break;
      default: XSetForeground(display,gc,foreground_pixel); break;
   }
}
