/***   axeshp.c  - subroutines for HP 2648 plots    ***/

#include <stdio.h>
#include <strings.h>
#include <math.h>
#include "header.h"

#define  HX_DIM   500
#define  HY_DIM   400
#define  HBDR_DIM  50
#define  HP_LIN     9
#define  HP_MKR     5
double   Hxmin, Hymin, Hxmax, Hymax;

/* draw the axes */
axeshp()
{
   char           *sprintf();
   extern char    xlabel[MAXCHAR];
   extern char    ylabel[MAXCHAR];
   extern char    toplabel[MAXCHAR];
   extern int     autorange,equalscale;
   extern double  xmin, ymin, xmax, ymax;

   double  xwid,ywid,ratio;

   /* Initialize */
   Hxmin = HBDR_DIM;
   Hymin = HBDR_DIM;
   xwid  = HX_DIM - 2*HBDR_DIM;
   ywid  = HY_DIM - 2*HBDR_DIM;
   Hxmax = Hxmin + xwid;
   Hymax = Hymin + ywid; 
   if (equalscale == ON) {
      /* max xwid = 400. max ywid = 300 */
      ratio = (ymax-ymin)/(xmax-xmin);
      if (ratio <= (ywid/xwid)) {
         Hxmax = Hxmin + xwid;
         Hymax = Hymin + ratio*xwid;
      } else {
         Hymax = Hymin + ywid;
         Hxmax = Hxmin + ywid/ratio;
      }
   }

   /* Turn on graphics display, set line type = 1 */
   printf("%c*dfz%c*dcz%c*m1b1q3x\n",esc,esc,esc);

   /* Define a grid pattern */
   printf("%c*m 8 1 cz\n",esc);

   /* define the axes */
   printf("%c*paf %6f %6f %6f %6f z\n",esc,Hxmin,Hymin,Hxmin,Hymax);
   printf("%c*paf %6f %6f %6f %6f z\n",esc,Hxmin,Hymax,Hxmax,Hymax);
   printf("%c*paf %6f %6f %6f %6f z\n",esc,Hxmax,Hymax,Hxmax,Hymin);
   printf("%c*paf %6f %6f %6f %6f z\n",esc,Hxmax,Hymin,Hxmin,Hymin);
   printf("%c*paf %6f %6f %6f %6f z\n",esc,Hxmin-1,Hymin-1,Hxmin-1,Hymax+1);
   printf("%c*paf %6f %6f %6f %6f z\n",esc,Hxmin-1,Hymax+1,Hxmax+1,Hymax+1);
   printf("%c*paf %6f %6f %6f %6f z\n",esc,Hxmax+1,Hymax+1,Hxmax+1,Hymin-1);
   printf("%c*paf %6f %6f %6f %6f z\n",esc,Hxmax+1,Hymin-1,Hxmin-1,Hymin-1);

   /* Set the options for the labels */
   printf("%c*m 1n4q7x\n",esc);

   /* Top Label */
   printf("%c*dsk %6f %6f oz\n",esc,0.5*(Hxmax+Hxmin),Hymax+10);
   printf("%c*dS%s\n",esc,toplabel);

   /* Set the options for the labels */
   printf("%c*m 1n1q3x\n",esc);

   /* Draw the tick marks and labels */
   if (!autorange) {
      plot_HPticks();
   } else {
      plot_HPAticks();
   }

   /* Set the options for the labels */
   printf("%c*m 1n6q7x\n",esc);

   /* X-axis Label */
   printf("%c*dsk %6f %6f oz\n",esc,0.5*(Hxmax+Hxmin),Hymin-20);
   printf("%c*dS%s\n",esc,xlabel);

   /* Set the options for the labels */
   printf("%c*m 2n4q7x\n",esc);

   /* Y-axis Label */
   printf("%c*dsk %6f %6f oz\n",esc,Hxmin-40,0.5*(Hymin+Hymax));
   printf("%c*dS%s\n",esc,ylabel);

   /* Turn off graphics text mode */
   printf("%c*dt\n",esc);

}

/* Draw the tick marks and labels */
plot_HPticks()
{
   char           *sprintf();
   extern int     grid,xticks,yticks,xlog,ylog;
   extern double  scale;
   extern double  xmin, xmax, ymin, ymax;

   double xdx,ydy,xintv,yintv;
   double Hxtmp,Hytmp;
   double vallbl;
   int    i;

   /* reset the number of ticks */
   if (xlog) {
      xticks = (int)(xmax - xmin - 1e-5) + 1;
      xmax   = xmin + xticks;
   }
   if (ylog) {
      yticks = (int)(ymax - ymin - 1e-5) + 1;
      ymax   = ymin + yticks;
   }

   /* Useful data */
   xdx   = (xmax-xmin)/xticks;
   ydy   = (ymax-ymin)/yticks;
   xintv = (Hxmax-Hxmin)/xticks;
   yintv = (Hymax-Hymin)/yticks;

   /* Tick Marks on the X-axis */
   for (i=0; i<=xticks; i++) {
      Hxtmp = Hxmin + xintv*i;
      Hytmp = Hymin;
      printf("%c*paf %6f %6f %6f %6f z\n",esc,Hxtmp,Hytmp,Hxtmp,Hytmp+3);

      vallbl = xmin + i*xdx;
      plot_HPxticklabel(vallbl,Hxtmp);

      Hytmp = Hymax;
      printf("%c*paf %6f %6f %6f %6f z\n",esc,Hxtmp,Hytmp-3,Hxtmp,Hytmp);
   }

   /* Tick Marks on the Y-axis */
   for (i=0; i<=yticks; i++) {
      Hytmp = Hymin + yintv*i;
      Hxtmp = Hxmin;
      printf("%c*paf %6f %6f %6f %6f z\n",esc,Hxtmp,Hytmp,Hxtmp+3,Hytmp);

      vallbl = ymin + i*ydy;
      plot_HPyticklabel(vallbl,Hytmp);

      Hxtmp = Hxmax;
      printf("%c*paf %6f %6f %6f %6f z\n",esc,Hxtmp-3,Hytmp,Hxtmp,Hytmp);
   }

   if (grid == ON) {
      for (i=0; i<=xticks; i++) {
         Hxtmp = Hxmin + xintv*i;
         linetyphp(3);
         printf("%c*paf %6f %6f %6f %6f z\n",esc,Hxtmp,Hymin+3,Hxtmp,Hymax-3);
         linetyphp(1);
      }
      for (i=0; i<=yticks; i++) {
         Hytmp = Hymin + yintv*i;
         linetyphp(3);
         printf("%c*paf %6f %6f %6f %6f z\n",esc,Hxmin+3,Hytmp,Hxmax-3,Hytmp);
         linetyphp(1);
      }
   }
}

/* Draw the tick marks and labels with automatic ranging */
plot_HPAticks()
{
   char           *sprintf();
   double         log10();
   extern int     grid,xticks,yticks,xlog,ylog;
   extern double  scale;
   extern double  xmin, xmax, ymin, ymax;

   double xmin2, xmax2, deltx, ymin2, ymax2, delty;
   double xtmp, ytmp, xt, yt, xfactor, yfactor;
   double Hxtmp,Hytmp;
   int    i, nticks;

   /*
    * useful data
    * X = Hxmin + (x-xmin)*(Hxmax-Hxmin)/(xmax-xmin)
    * Y = Hymin + (y-ymin)*(Hymax-Hymin)/(ymax-ymin)
    */
   xfactor = (Hxmax - Hxmin)/(xmax-xmin);
   yfactor = (Hymax - Hymin)/(ymax-ymin);

   /* get rounded limits */
   get_autorange(xmin,xmax,ymin,ymax,
                 &xmin2,&xmax2,&deltx,&ymin2,&ymax2,&delty);

   /* Tick Marks on the X-axis */

   /* draw minor tick marks */
   xtmp = xmin2;
   while (xtmp < xmax) {
      for (i=1; i<10; i++) {
         xt = xtmp + i*0.1*deltx;
         if (xlog) xt = xtmp + log10((double)i);
         if ((xt > xmin) && (xt < xmax)) {
            Hxtmp = Hxmin + (xt-xmin)*xfactor;
            if (i==5) {
            printf("%c*paf %6f %6f %6f %6f z\n",esc,Hxtmp,Hymin,Hxtmp,Hymin+3);
            printf("%c*paf %6f %6f %6f %6f z\n",esc,Hxtmp,Hymax-3,Hxtmp,Hymax);
            } else {
            printf("%c*paf %6f %6f %6f %6f z\n",esc,Hxtmp,Hymin,Hxtmp,Hymin+1);
            printf("%c*paf %6f %6f %6f %6f z\n",esc,Hxtmp,Hymax-1,Hxtmp,Hymax);
            }
         }
      }
      xtmp += deltx;
   }

   /* major tickmarks and accompanying labels */
   xtmp = xmin2 + deltx;
   nticks = 0;
   while (xtmp < xmax-0.0001) {
      Hxtmp = Hxmin + (xtmp-xmin)*xfactor;
      printf("%c*paf %6f %6f %6f %6f z\n",esc,Hxtmp,Hymin,Hxtmp,Hymin+4);
      printf("%c*paf %6f %6f %6f %6f z\n",esc,Hxtmp,Hymax-4,Hxtmp,Hymax);
      plot_HPxticklabel(xtmp,Hxtmp);
      xtmp += deltx;
      nticks ++;
   }

   /* put labels on the min and max points */
   if ((nticks==0) || ((xmin - xmin2) < 0.5*deltx)) {
      plot_HPxticklabel(xmin,Hxmin);
   }

   /* put labels on the min and max points */
   if ((nticks==0) || ((xtmp - xmax) < 0.5*deltx)) {
      plot_HPxticklabel(xmax,Hxmax);
   }

   /* Tick Marks on the Y-axis */

   /* draw minor tick marks */
   ytmp = ymin2;
   while (ytmp < ymax) {
      for (i=1; i<10; i++) {
         yt = ytmp + i*0.1*delty;
         if (ylog) yt = ytmp + log10((double)i);
         if ((yt > ymin) && (yt < ymax)) {
            Hytmp = Hymin + (yt-ymin)*yfactor;
            if (i==5) {
            printf("%c*paf %6f %6f %6f %6f z\n",esc,Hxmin,Hytmp,Hxmin+3,Hytmp);
            printf("%c*paf %6f %6f %6f %6f z\n",esc,Hxmax-3,Hytmp,Hxmax,Hytmp);
            } else {
            printf("%c*paf %6f %6f %6f %6f z\n",esc,Hxmin,Hytmp,Hxmin+1,Hytmp);
            printf("%c*paf %6f %6f %6f %6f z\n",esc,Hxmax-1,Hytmp,Hxmax,Hytmp);
            }
         }
      }
      ytmp += delty;
   }

   /* major tickmarks and accompanying labels */
   ytmp = ymin2 + delty;
   nticks = 0;
   while (ytmp < ymax-0.0001) {
      Hytmp = Hymin + (ytmp-ymin)*yfactor;
      printf("%c*paf %6f %6f %6f %6f z\n",esc,Hxmin,Hytmp,Hxmin+3,Hytmp);
      printf("%c*paf %6f %6f %6f %6f z\n",esc,Hxmax-3,Hytmp,Hxmax,Hytmp);
      plot_HPyticklabel(ytmp,Hytmp);
      ytmp += delty;
      nticks ++;
   }

   /* put labels on the min and max points */
   if ((nticks==0) || ((ymin - ymin2) < 0.5*delty)) {
      plot_HPyticklabel(ymin,Hymin);
   }

   /* put labels on the min and max points */
   if ((nticks==0) || ((ytmp - ymax) < 0.5*deltx)) {
      plot_HPyticklabel(ymax,Hymax);
   }

   /* draw the grid */
   if (grid == ON) {
      xtmp = xmin2 + deltx;
      while (xtmp < xmax - 0.0001) {
         Hxtmp = Hxmin + (xtmp-xmin)*xfactor;
         linetyphp(3);
         printf("%c*paf %6f %6f %6f %6f z\n",esc,Hxtmp,Hymin+3,Hxtmp,Hymax-3);
         linetyphp(1);
         xtmp += deltx;
      }
      ytmp = ymin2 + delty;
      while (ytmp < ymax - 0.0001) {
         Hytmp = Hymin + (ytmp-ymin)*yfactor;
         linetyphp(3);
         printf("%c*paf %6f %6f %6f %6f z\n",esc,Hxmin+3,Hytmp,Hxmax-3,Hytmp);
         linetyphp(1);
         ytmp += delty;
      }
   }
}

/* plot the tick label on the x-axis */
plot_HPxticklabel(vallbl,Hxtmp)
double vallbl, Hxtmp;
{
   char   *sprintf();
   double pow();
   extern int     xlog;

   char   label[MAXCHAR];

   if (xlog) vallbl = pow(10.0,vallbl);
   sprintf(label,"%.3g",vallbl);
   printf("%c*paf %6f %6f z\n",esc,Hxtmp-35,Hymin-15);
   printf("%c*l%s\n",esc,label);
}

/* plot the tick label on the y-axis */
plot_HPyticklabel(vallbl,Hytmp)
double vallbl, Hytmp;
{
   char   *sprintf();
   double pow();
   extern int     xlog;

   char   label[MAXCHAR];

   if (ylog) vallbl = pow(10.0,vallbl);
   sprintf(label,"%.3g",vallbl);
   printf("%c*paf %6f %6f z\n",esc,Hxmin-55,Hytmp);
   printf("%c*l%s\n",esc,label);
}


/* plot the curves */
plothp()
{
   extern int     line, marker;
   extern int     linechange, markerchange;
   extern int     linetype[MAXTYPE], markertype[MAXTYPE], plottypes;
   extern double  xmin, xmax, ymin, ymax;
   extern segmptr segmhead;

   segmptr  S;
   nodeptr  N;
   double   x,y,idx,jdy;
   int      kl,km,nplt=0;

   idx = (Hxmax-Hxmin)/(xmax-xmin);
   jdy = (Hymax-Hymin)/(ymax-ymin);

   kl  = linetype[nplt];
   km  = markertype[nplt];

   S = segmhead;
   for ( ; S!=NULL; S=S->next) {
      /* draw the line */
      if (line == ON && kl!=0) {
         linetyphp(kl);
         printf("%c*paf \n",esc);     /* new line */
         N = S->head;
         for ( ; N!=NULL; N=N->next) {
            x = (N->x - xmin)*idx + Hxmin;
            y = (N->y - ymin)*jdy + Hymin;
            printf("%6f %6f \n",x,y);
         }
         printf(" z\n");
      }

      /* plot markers */
      N = S->head;
      if (marker==ON || (kl==0 && km==0)) {
         linetyphp(1);
         km = km % HP_MKR;
         for ( ; N!=NULL; N=N->next) {
            x = (N->x - xmin)*idx + Hxmin;
            y = (N->y - ymin)*jdy + Hymin;
            switch (km) { 
            case 0: printf("%c*paf %6f %6f ",esc,x-1,y-1); 
                    printf("%6f %6f z\n",        x+1,y+1); 
                    printf("%c*paf %6f %6f ",esc,x+1,y-1); 
                    printf("%6f %6f z\n",        x-1,y+1); 
                    break;                   /* diagonal cross */
            case 1: printf("%c*paf %6f %6f z\n",esc,x,y);
                    break;                   /* dot (hard to see) */
            case 2: printf("%c*paf %6f %6f ",esc,x-1,y); 
                    printf("%6f %6f z\n",        x+1,y); 
                    printf("%c*paf %6f %6f ",esc,x,y-1); 
                    printf("%6f %6f z\n",        x,y+1); 
                    break;                         /* vertical cross */
            case 3: printf("%c*paf %6f %6f ",esc,x-1,y-1); 
                    printf("%6f %6f ",           x+1,y-1); 
                    printf("%6f %6f ",           x+1,y+1); 
                    printf("%6f %6f ",           x-1,y+1); 
                    printf("%6f %6f z\n",        x-1,y-1); 
                    break;                          /* square */
            case 4: printf("%c*paf %6f %6f ",esc,x  ,y-1); 
                    printf("%6f %6f ",           x+1,y  ); 
                    printf("%6f %6f ",           x  ,y+1); 
                    printf("%6f %6f ",           x-1,y  ); 
                    printf("%6f %6f z\n",        x  ,y-1); 
                    break;
            default:break;
            }
         }
      }
      /* increment the plot-type by increasing the array index */
      nplt++;
      if (nplt >= plottypes) nplt = nplt % plottypes;
      /* now look up the line and marker type */
      if (linechange   == ON) kl = linetype[nplt];
      if (markerchange == ON) km = markertype[nplt];
   }
}

/* change the linetype */
linetyphp(line)
int line; 
{
   int newline, newcolor;

   line = line % HP_LIN;
   switch (line) {
   case 0 : newline =11; newcolor = 7; break;
   case 1 : newline = 1; newcolor = 3; break;
   case 2 : newline = 8; newcolor = 4; break;
   case 3 : newline = 7; newcolor = 3; break;
   case 4 : newline = 5; newcolor = 5; break;
   case 5 : newline = 4; newcolor = 6; break;
   case 6 : newline = 6; newcolor = 7; break;
   case 7 : newline = 9; newcolor = 1; break;
   case 8 : newline =10; newcolor = 2; break;
   }

   printf("%c*m%db%dxz\n",esc,newline,newcolor);

   /* newline =  1 : solid line             color = 0 : black      */
   /* newline =  4 : dot-dash               color = 1 : red        */
   /* newline =  5 : long dash              color = 2 : green      */
   /* newline =  6 : short dash             color = 3 : yellow     */
   /* newline =  7 : dots                   color = 4 : dark blue  */
   /* newline =  8 : dot-dash               color = 5 : purple     */
   /* newline =  9 : 3-dash                 color = 6 : light blue */
   /* newline = 10 : 3-dash                 color = 7 : white      */
   /* newline = 11 : point plot                                    */
}

/* end procedures - reset the terminal */
endgrhp()
{
   printf("%c*delt %c*m1b7xZ\n",esc,esc);
}

